/*

*/

import tree.*;
import tokenizer.*;
import treelanguage.*;
import language3.*;

/**
   This program parses and then evaluates strings from Language_3.
*/
public class Language_3_Examples
{
   public static void main(String[] args)
   {
      // IMPORTANT: Set this to 0 or 1 depending on whether you need
      // to see all of the interpreter's debugging information.
      EvaluateAST.DEBUG = 1;

      String[] programs =
      {
         String.join("\n",
         "(prog",
         "  (begin (var x 2)",
         "         (var y 2))",
         "  (var z (+ x y)))"),

         String.join("\n",
         "(prog",
         "  (var x 0)",
         "  (var y 0)",
         "  (begin (var x 2)",
         "         (var y 2))",
         "         (set x (+ x y 3)))"),

         String.join("\n",
         "(prog",
         "  (var x 4)",
         "  (begin (var y false)",
         "         (|| y (== (+ 2 x) (* 2 3)))))"),

         String.join("\n",
         "(prog",
         "  (var x1 1)",
         "  (set x1 (+ x1 2))",
         "  (begin",
         "    (set x1 (+ x1 2))",
         "    (var x1 (+ x1 2))",
         "    (set x1 (+ x1 2)))",
         "  (set x1 (+ x1 2))",
         "  (var x2 (+ x1 2)))"),

         String.join("\n",
         "(prog",
         "  (var x 1)",
         "  (begin",
         "    (print x)",
         "    (var x 2)",
         "    (begin",
         "      (print x)",
         "      (set x 3)",
         "      (print x)",
         "      (var x 4))",
         "    (print x)",
         "    (set x 5))",
         "  (print x))"),

         String.join("\n",
         "(prog",
         "  (var n 1)",
         "  (var x1 (* n n))",
         "  (set n 2)",
         "  (var x2 (* n n))",
         "  (set n 3)",
         "  (var x3 (* n n))",
         "  (set n 4)",
         "  (var x4 (* n n))",
         "  (+ (+ (+ x1 x2) x3) x4))")
      };

      int i = 0;
      for (final String str : programs)
      {
         System.out.println("\n" + i + " =========================================");

         // Build and evaluate the abstract syntax tree that represents the program.
         try
         {
            final Tree<Token> ast = Parse2AST.parse( str );

            // Print the AST as an S-expression
            System.out.println( EvaluateAST.toString(ast) + "\n" );

            // Pretty-print the abstract syntax tree.
            //System.out.println( PrettyPrintAST.prettyPrint( ast ) + "\n" );

            System.out.println("// Unparsed program: " );
            // Print the infix version of the expression.
            System.out.println( AST2infix.ast2infix( ast ) + "\n" );

            // Create dot and png files from the AST.
            final String baseName = String.format("Language_3.%02d", i);
            Tree2dot.tree2dot(ast, baseName);
            Tree2png.tree2png(baseName);

            // Create an xml file from the AST.
            AST2xml.ast2xml(ast, baseName);

            // Run the program (interpret the AST).
            final Value value = EvaluateAST.eval( ast );
            if (EvaluateAST.DEBUG > 0)
            {
               System.out.println("result = " + value + "\n" );
            }
            else
            {
               System.out.println("result = " + value.toSimpleString() + "\n" );
            }
         }
         catch (TokenizeError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
          //e.printStackTrace();
         }
         catch (ParseError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
          //e.printStackTrace();
         }
         catch (EvalError e)
         {
            System.out.println( " ... Failed (" + e + ")" );
          //e.printStackTrace();
         }
         ++i;
      }
   }
}
